﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace AnalysisApplication
{
    public class FileViewVisitor : DefaultModelVisitor
    {
        private List<IVisitorObserver> tenMinuteObservers;
        private List<IVisitorObserver> summaryObservers;

        private DateTime startTenMinuteBlock;
        private List<string> filesViewedTenMinutes;

        private List<string> filesViewedSession;

        public FileViewVisitor()
        {
            tenMinuteObservers = new List<IVisitorObserver>();
            summaryObservers = new List<IVisitorObserver>();

            filesViewedSession = new List<string>();
            filesViewedTenMinutes = new List<string>();
        }
        #region visit    
    
        public override void VisitSessionModel(SessionModel m)
        {
            startTenMinuteBlock = m.start;
        }

        public override void VisitFileView(FileView f)
        {
            if (f.Timestamp.Subtract(startTenMinuteBlock).Minutes > 10)
            {
                notifyTenMinuteObs();
                startTenMinuteBlock = f.Timestamp;
            }            
            filesViewedSession.Add(f.File);
            filesViewedTenMinutes.Add(f.File);
        }

        #endregion visit
        #region Observed subject
        #region attach observers
        public override void Attach10MinuteObserver(IVisitorObserver v)
        {
            tenMinuteObservers.Add(v);
        }

        public override void AttachSummaryObserver(IVisitorObserver v)
        {
            summaryObservers.Add(v);
        }
        #endregion attach observers
        #region Retrieve data from visitor
        public override Dictionary<string, string> GetSessionData()
        {
            Dictionary<string, string> results = new Dictionary<string, string>();
            results.Add("NumberFilesViewed", filesViewedSession.Count.ToString());
            HashSet<string> fileSet = new HashSet<string>();
            foreach (string s in filesViewedSession)
            {
                fileSet.Add(s);
            }
            results.Add("NumberUniqueFilesViewed", fileSet.Count.ToString());
            return results;
        }

        public override Dictionary<string, string> Get10MinuteData()
        {
            Dictionary<string, string> results = new Dictionary<string, string>();
            results.Add("NumberFilesViewed", filesViewedTenMinutes.Count.ToString());
            HashSet<string> fileSet = new HashSet<string>();
            foreach (string s in filesViewedTenMinutes){
                fileSet.Add(s);
            }
            results.Add("NumberUniqueFilesViewed", fileSet.Count.ToString());
            return results;
        }
        #endregion Retrieve data from visitor

        #region notify
        private void notifyTenMinuteObs()
        {
            foreach (IVisitorObserver o in tenMinuteObservers)
            {
                o.UpdateSummary(this);
            }
            filesViewedTenMinutes.Clear();
        }

        private void notifySessionObs()
        {
            foreach (IVisitorObserver o in summaryObservers)
            {
                o.UpdateSummary(this);
            }
            filesViewedSession.Clear();
        }
        #endregion notify
        #endregion Observed subject

        public override void ModelElementEnd(ModelElement e)
        {
            if (e is SessionModel)
            {
                // Are at the end of processing a session. 
                notifySessionObs();
            }
        }
    }
}
