﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace AnalysisApplication
{
    public class ProjectSession : ModelElement
    {
        public DateTime start;
        public DateTime end;
        public string file;

        public string project;
        public List<LowLevelModelElement> sessionActions;

        private ProjectLaunch currentLaunch;
        private FileView currentView;

        public ProjectSession(IDEEvent e)
        {
            sessionActions = new List<LowLevelModelElement>();
            start = e.timestamp;
            project = e.projectName;
            file = e.fileLocation;
        }

        public void CloseProject(IDEEvent e)
        {
            end = e.timestamp;
        }

        public void ProcessEvent(IDEEvent e)
        {
            if (e.type.Contains("File"))
            {
                if (e.action == "Start")
                {
                    currentView = new FileView(e);
                    sessionActions.Add(currentView);
                }
                else
                {
                    if (currentView != null)
                    {
                        currentView.FileViewStopped(e);
                    }
                }
            }
            else if (e.type.Contains("Run"))
            {
                if (e.action.Contains("start"))
                {
                    currentLaunch = new ProjectLaunch(e);
                    sessionActions.Add(currentLaunch);
                }
            }
            else if (e.type.Contains("Console") && currentLaunch != null)
            {
                currentLaunch.AddOutput(e);
            }
            
        }

        public override void Accept(DefaultModelVisitor v)
        {
            v.VisitProjectSession(this);
            foreach (LowLevelModelElement m in sessionActions)
            {
                m.Accept(v);
            }
        }

        public void EndEvent(IDEEvent e)
        {
            end = e.timestamp;
        }
    }
}
