﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace AnalysisApplication
{
    public class SessionModel : ModelElement//, IEnumerable<ModelElement>
    {


        public string user;
        public string operatingSystem;
        public string iPAddress;

        public List<ProjectSession> projectSessions;

        public DateTime start;
        public DateTime end;

        public SessionModel(List<IDEEvent> events)
        {
            projectSessions = new List<ProjectSession>();
            ProcessEvents(events);
        }

        private void ProcessEvents(List<IDEEvent> events)
        {
            ProjectSession currentPS = null;
            events.Sort();//.Sort(CompareEvents);
            foreach (IDEEvent e in events)
            {
                if (e.type != null && e.type.Contains("Project"))
                {
                    if (e.action.Contains("Start"))
                    {
                        currentPS = new ProjectSession(e);
                        projectSessions.Add(currentPS);
                    }
                    else
                    {
                        currentPS.EndEvent(e);
                    }
                }
                else if (e.type.Contains("Session"))
                {
                    if (e.action.Contains("Start"))
                    {
                        start = e.timestamp;
                    }
                    else if (e.type.Contains("Details"))
                    {
                        operatingSystem = e.stringData["OperatingSystem"];
                        user = e.stringData["User"];
                        iPAddress = e.stringData["IPAddress"];
                    }
                    else
                    {
                        end = e.timestamp;
                    }
                }
                else if (currentPS != null)
                {
                    currentPS.ProcessEvent(e);
                }
            }
        }

        public override void Accept(DefaultModelVisitor v)
        {
            v.VisitSessionModel(this);
            foreach (ProjectSession p in projectSessions)
            {
                p.Accept(v);
            }
            v.ModelElementEnd(this);
        }

    }
}
