﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml;

namespace AnalysisApplication
{
    public class XMLEventLoader : FileEventLoader
    {
        private FileInfo[] filesToLoad;
        private System.Collections.IEnumerator fileInfoIterator;
        private string processDirectory = @"H:\Share";

        public XMLEventLoader(string directory)
        {
            processDirectory = directory; 
        }

        public override bool Initialise()
        {
            DirectoryInfo MyRoot = new DirectoryInfo(processDirectory);
            try
            {
                filesToLoad = MyRoot.GetFiles("*.xml", SearchOption.AllDirectories);
                fileInfoIterator = filesToLoad.GetEnumerator();
            }
            catch (DirectoryNotFoundException e)
            {
                return false;
            }
            return true;
        }

        public override List<IDEEvent> NextSession()
        {
            if (fileInfoIterator.MoveNext())
            {
                return MakeSession((FileInfo)fileInfoIterator.Current);
            }
            return null;
        }

        public override string ProcessDirectory
        {
            get { return processDirectory; }
        }

        private List<IDEEvent> MakeSession(FileInfo file)
        {
            XmlReader reader = new XmlTextReader(new StreamReader(file.FullName));           
            return ProcessSession(reader);
        }

        private List<IDEEvent> ProcessSession(XmlReader reader)
        {
            List<IDEEvent> events = new List<IDEEvent>();
            string s;
            try
            {
                while (reader.Read())
                {
                    if(reader.NodeType == XmlNodeType.Element)
                    {
                        if (reader.LocalName.Contains("SESSION"))
                        {
                            IDEEvent e = new IDEEvent();
                            e.kind = "Details";
                            e.type = "Session Details";
                            e.action = "";
                            e.stringData.Add("User", reader.GetAttribute("User"));
                            e.stringData.Add("OperatingSystem", reader.GetAttribute("OperatingSystem"));
                            e.stringData.Add("IPAddress", reader.GetAttribute("IPAddress"));
                            events.Add(e);
                        }
                        else if (reader.LocalName.Contains("EVENT"))
                        {
                            IDEEvent e = new IDEEvent();
                            e.kind = reader.GetAttribute("Kind");
                            e.type = reader.GetAttribute("Type");
                            e.action = reader.GetAttribute("Action");
                            e.timestamp = DateTime.Parse(reader.GetAttribute("Timestamp"));
                            e.fileLocation = reader.GetAttribute("FileFocus");

                            while (reader.Read())
                            {
                                if (reader.NodeType == XmlNodeType.Element &&
                                    reader.LocalName.Contains("MODEL"))
                                {
                                    e.projectName = reader.GetAttribute("ProjectName");
                                    e.packageName = reader.GetAttribute("PackageName");
                                    e.className = reader.GetAttribute("ClassName");
                                    e.methodName = reader.GetAttribute("MethodName");
                                    s = reader.GetAttribute("LineNumber");
                                    if (s != "")
                                    {
                                        e.lineNumber = int.Parse(s);
                                    }
                                    s = reader.GetAttribute("RevisionNumber");
                                    if (s != "")
                                    {
                                        e.revisionNumber = int.Parse(s);
                                    }
                                }
                                else if (reader.LocalName == "PARAM")
                                {
                                    if (reader.GetAttribute("Type") == "String")
                                    {
                                        e.stringData.Add(reader.GetAttribute("Name"),
                                            reader.GetAttribute("Value"));
                                    }
                                    else if (reader.GetAttribute("Type") == "Double")
                                    {
                                        e.doubleData.Add(reader.GetAttribute("Name"),
                                            double.Parse(reader.GetAttribute("Value")));
                                    }
                                }
                                else if (reader.NodeType == XmlNodeType.EndElement
                                    && reader.LocalName.Contains("EVENT"))
                                {
                                    events.Add(e);
                                    break;
                                }
                            }
                        }                        
                    }
                }
            }
            catch (XmlException e)
            {
            }
            return events;
        }
    }
}
