﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using ICSharpCode.SharpZipLib.Zip;

namespace AnalysisApplication
{
    public class ZipFileEventLoader : FileEventLoader
    {
        FileEventLoader fileLoader;
        private string destinationDirectory;
        private string processDirectory;

        public ZipFileEventLoader(FileEventLoader fileLoader, string processDirectory)
        {
            this.fileLoader = fileLoader;
            this.processDirectory = processDirectory;
        }

        public override bool Initialise()
        {
            destinationDirectory = fileLoader.ProcessDirectory;
            bool outcome = ExtractFiles();
            outcome = outcome && fileLoader.Initialise();
            return outcome;
        }

        public override List<IDEEvent> NextSession()
        {
            return fileLoader.NextSession();
        }

        public override string ProcessDirectory
        {
            get { return processDirectory; }
        }

        /**
        * Extracts all the files in the UPLOAD_DIR into the PROCESS_DIR
        */
        private bool ExtractFiles()
        {
            DirectoryInfo dir = new DirectoryInfo(ProcessDirectory);
            FileInfo[] MyFiles = getFilesInDirectory(dir);
            foreach (FileInfo fi in MyFiles)
            {
                UnZipFile(fi, destinationDirectory);
            }            
            return true;
        } 

        /**
        * Gets information about all the files in a given directory.
        */
        private FileInfo[] getFilesInDirectory(DirectoryInfo dir)
        {
            FileInfo[] MyFiles = dir.GetFiles("*.zip",SearchOption.AllDirectories);
            return MyFiles;
        }

        /**
        * Gets information about all the files in a given directory. 
        */
        private DirectoryInfo[] getFoldersInDirectory(string dirName)
        {
            DirectoryInfo MyRoot = new DirectoryInfo(dirName);
            DirectoryInfo[] MyDir = MyRoot.GetDirectories();
            return MyDir;
        }

        /**
        * Extracts the given file into the directory provided.
        */
        private bool UnZipFile(FileInfo Input, string baseDirectory)
        {
            bool ret = true;
            if (!Directory.Exists(baseDirectory))
            {
                Directory.CreateDirectory(baseDirectory);
            }
            try
            {
                if (Input.Exists)
                {
                    using (ZipInputStream ZipStream = new
                        ZipInputStream(Input.OpenRead()))
                    {
                        ZipEntry theEntry;
                        while ((theEntry = ZipStream.GetNextEntry()) != null)
                        {
                            if (theEntry.IsFile)
                            {
                                if (theEntry.Name != "")
                                {
                                    string strNewFile = @"" + baseDirectory + @"\" +
                                        theEntry.Name;
                                    if (File.Exists(strNewFile))
                                    {
                                        continue;
                                    }

                                    using (FileStream streamWriter = File.Create(strNewFile))
                                    {
                                        int size = 2048;
                                        byte[] data = new byte[2048];
                                        while (true)
                                        {
                                            size = ZipStream.Read(data, 0, data.Length);
                                            if (size > 0)
                                                streamWriter.Write(data, 0, size);
                                            else
                                                break;
                                        }
                                        streamWriter.Close();
                                    }
                                }
                            }
                            else if (theEntry.IsDirectory)
                            {
                                string strNewDirectory = @"" + baseDirectory + @"\" +
                                    theEntry.Name;
                                if (!Directory.Exists(strNewDirectory))
                                {
                                    Directory.CreateDirectory(strNewDirectory);
                                }
                            }
                        }
                        ZipStream.Close();
                    }
                }
            }
            catch (Exception)
            {
                ret = false;
            }
            return ret;
        }
    }
}
